import shlex
import subprocess
from typing import List, Sequence, Optional, Tuple

#
# Construct the smartctl/btrfs/...  commands safely.
#

def build_smartctl_scan(device: str) -> List[str]:
    """
    Get disks available:
    """
    return shlex.split(f"smartctl --scan {shlex.quote(device)}")

def build_sudo_smartctl_a(device: str) -> List[str]:
    """
    Legacy way to retrieve info
    """
    return shlex.split(f"sudo smartctl -a {shlex.quote(device)}")

def build_sudo_smartctl_l_error(device: str) -> List[str]:
    """
    retrieve errors
    """
    return shlex.split(f"sudo smartctl -l error {shlex.quote(device)}")

def build_sudo_smartctl_x(device: str) -> List[str]:
    """
    SMART overall-health self-assessment test result:
    test in progress
    completed test in power_on_hours
    """
    return shlex.split(f"sudo smartctl -x {shlex.quote(device)}")

def build_sudo_smartctl_t_short(device: str) -> List[str]:
    """
    run short test
    """
    return shlex.split(f"sudo smartctl -t short {shlex.quote(device)}")

def build_sudo_smartctl_l_selftest(device: str) -> List[str]:
    """
    show self-test log (subset of -x )
    """
    return shlex.split(f"sudo smartctl -l selftest {shlex.quote(device)}")

def build_sudo_btrfs_filesystem_show() -> List[str]:
    """
     list all partitions with btrfs; last info in every paragraph
     is: "path /dev/nvme0n1p1"
    """
    return shlex.split(f"sudo btrfs filesystem show")

def build_sudo_btrfs_scrub_status(mountpoint: str) -> List[str]:
    """
     Show if scrub is running. Show errors.
    """
    return shlex.split(f"sudo btrfs scrub status {shlex.quote(mountpoint)}")

def build_sudo_btrfs_scrub_start(mountpoint: str) -> List[str]:
    """
     Start scrub if it is not already running.
    """
    return shlex.split(f"sudo btrfs scrub start {shlex.quote(mountpoint)}")

def build_sudo_btrfs_device_stats(device: str) -> List[str]:
    """
     Show if scrub is running. Show errors.
    """
    return shlex.split(f"sudo btrfs device stats {shlex.quote(device)}")

def build_snapper_list_configs() -> List[str]:
    """
    Snapper operated btrfs mountpoints:
    Drop 1st 3 lines, the rest is rows with:  configname | mountpoint
    """
    return shlex.split(f"snapper list-configs")

def build_sudo_snapper_c_list(configname: str) -> List[str]:
    """
    list all snapshots
    """
    return shlex.split(f"sudo snapper -c {shlex.quote(configname)} list")



def run_command(command: Sequence[str], timeout: int = 10) -> Tuple[int, str]:
    """Execute a command and return (returncode, output).

    Args:
        command: List of command arguments
        timeout: Maximum time to wait for command completion (seconds)

    Returns:
        Tuple of (returncode, combined_stdout_stderr)
    """
    try:
        result = subprocess.run(
            list(command),
            capture_output=True,
            text=True,
            timeout=timeout,
            check=False,
        )
        return result.returncode, result.stdout + result.stderr
    except subprocess.TimeoutExpired:
        return 1, f"Command timed out after {timeout} seconds"
    except Exception as e:
        return 1, str(e)



#!/usr/bin/env python3
import os
import re
import subprocess
import sys
import getpass

import click
import pandas as pd

import re
from console import fg, bg
import sys

_ANSIRE = re.compile(r'\x1b\[[0-9;]*m')

ESC = "\033["
RESET = ESC + "0m"
GREEN = ESC + "32m"
RED = ESC + "31m"
YELLOW = ESC + "33m"
WHITE = ESC + "37m"

"""
make this in
 sudo visudo -f /etc/sudoers.d/ufw-status
youruser ALL=(root) NOPASSWD: /usr/sbin/ufw status, /usr/sbin/ufw status verbose
"""

def strip_ansi(s):
    return _ANSIRE.sub('', s)

def pad_ansi(s, width, left=True):
    vis = strip_ansi(s)
    pad = max(0, width - len(vis))
    return (s + ' ' * pad) if left else (' ' * pad + s)

def run(cmd):
    p = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL, text=True)
    return p.stdout


def original_user():
    try:
        out = run(["logname"]).strip()
        if out:
            return out
    except Exception:
        pass
    # fallback
    return os.environ.get("SUDO_USER") or getpass.getuser()


def check_sudoers_file():
    path = '/etc/sudoers.d/ufw-status'
    if os.path.exists(path):
        print("exists")
        return True #sys.exit(0)
    print("X...", path, "not found")
    print("""
    sudo visudo -f /etc/sudoers.d/ufw-status
youruser ALL=(root) NOPASSWD: /usr/sbin/ufw status, /usr/sbin/ufw status verbose
    """)
    return sys.exit(1)


def parse_ufw():
    """Return dict port->rule_str and list of allowed numeric ports (unique, sorted)."""
    txt = run(["sudo", "ufw", "status"])

    rules = {}
    allowed = set()
    for line in txt.splitlines():
        line = line.strip()
        if not line:
            continue
        # skip lines that start with an IP (like "Anywhere  ALLOW IN  1.2.3.4")
        if re.match(r"^[0-9]+\.", line):
            continue
        parts = re.split(r"\s+", line)
        first = parts[0]
        # normalize first column removing /tcp /udp
        first_norm = re.sub(r"/(tcp|udp)$", "", first)
        if re.match(r"^\d+$", first_norm):
            rule_str = " ".join(parts[:4]) if len(parts) >= 4 else " ".join(parts)
            rules[first_norm] = rule_str
            if "DENY" not in rule_str and "DENY" not in line:
                allowed.add(int(first_norm))
    allowed_list = sorted(allowed)
    if len(rules) == 0 or len(allowed_list) == 0:
        # print(rules, allowed_list)
        print(f"X... {bg.red}{fg.white} UFW - SOME PROBLEM with FIREWALL {fg.default}{bg.default}")
    return rules, allowed_list


def check_ufw_for_port(rules, port):
    port_s = str(port)
    return rules.get(port_s, "")


def systemctl_status(svc):
    # append .service (systemctl handles either)
    unit = f"{svc}.service"
    out = run(["systemctl", "is-active", unit]).strip()
    return out or "unknown"


def show_warning_if_policy_not_deny():
    txt = run(["ufw", "status", "verbose"])
    if "Default: deny (incoming)" not in txt:
        click.echo("X... check default policies of ufw !!!")
        click.echo(txt)


def ps_contains(pattern):
    txt = run(["ps", "-ef"])
    return pattern in txt


def ss_listening_contains(port, name_substr=None):
    txt = run(["ss", "-tulpn"])
    if str(port) not in txt:
        return False
    if name_substr:
        return name_substr in txt
    return True


# ================================================================================
#
#                MAIN --------------------------
#
# --------------------------------------------------------------------------------
@click.command()
@click.option("-n", "--no-decoration", is_flag=True, help="Do not print headers / separators")
def main(no_decoration):
    user = run(["whoami"]).strip()
    orig_user = original_user()

    services = [
        ("ufw", 0),
        ("psad", 0),
        ("ssh", 22),
        ("influxdb", 8086),
        ("chrony", 323),
        ("ntp", 123),
        ("mosquitto", 1883),
        (f"syncthing@{orig_user}", 22000),
        ("grafana-server", 3000),
        ("smbd", 445),
        ("elog", 9000),
        ("nginx", 80),
        ("ntfy", 80),
    ]

    check_sudoers_file()
    rules, open_ports = parse_ufw()
    open_ports_left = list(open_ports)  # numeric list

    rows = []
    if not no_decoration:
        click.echo("  sysd-service       status            port        UFW-opened")
        click.echo("_______________________________________________________________")

    for svc, port in services:
        status = systemctl_status(svc)
        if status == "active":
            color = GREEN
        elif status == "inactive":
            color = WHITE
        else:
            color = RED
        ufw_status = ""
        ufw_color = GREEN
        if port != 0:
            ufw_status = check_ufw_for_port(rules, port)
            if "DENY" in ufw_status:
                ufw_color = RED
            # remove from open_ports_left if present
            if port in open_ports_left:
                open_ports_left = [p for p in open_ports_left if p != port]
        rows.append(
            {
                "service": svc,
                "status": f"{color}{status}{RESET}",
                "port": "" if port == 0 else str(port),
                "ufw": f"{ufw_color}{ufw_status}{RESET}" if ufw_status else "",
            }
        )

    # keep table in pandas
    df = pd.DataFrame(rows, columns=["service", "status", "port", "ufw"])

    # # print using our format to keep colors
    # for _, r in df.iterrows():
    #     svc = r["service"]
    #     status_colored = r["status"]
    #     port = r["port"]
    #     ufw = r["ufw"]
    #     if port:
    #         click.echo(f"  {svc:<20} {status_colored:<8}     Port {port:>5}   {ufw}")
    #     else:
    #         click.echo(f"  {svc:<20} {status_colored:<8}                  {ufw}")

    # printing loop (replace your current loop)
    for _, r in df.iterrows():
        svc = r["service"]
        status_colored = r["status"]
        port = r["port"]
        ufw = r["ufw"]

        svc_field = f"  {svc:<20}"
        status_field = pad_ansi(status_colored, 8)   # visible width 8
        if port:
            click.echo(f"{svc_field} {status_field}     Port {port:>5}   {ufw}")
        else:
            click.echo(f"{svc_field} {status_field}                  {ufw}")

    # TELEGRAF and VADIM checks (as in original)
    tlgf_active = False
    ps_txt = run(["ps", "-ef"])
    if f"{orig_user}.conf" in ps_txt and "telegraf" in ps_txt:
        tlgf_active = True
    vadi_active = ss_listening_contains(8200, "telegraf")

    tlgf_str = f"{GREEN}active{RESET}" if tlgf_active else f"{WHITE}inactive{RESET}"
    vadi_str = f"{GREEN}active{RESET}" if vadi_active else f"{WHITE}inactive{RESET}"

    click.echo(f" TELEGRAF@{orig_user}         {tlgf_str}")
    click.echo(f" VADIM-8200-{orig_user}       {vadi_str}")

    if not no_decoration:
        click.echo("_______________________________________________________________")

    if os.geteuid() == 0:
        click.echo("", nl=False)
        click.echo(f"{YELLOW}: {open_ports_left} {RESET}")
        show_warning_if_policy_not_deny()


if __name__ == "__main__":
    main()



"""Main CLI entry point for the disk manager application."""

import click
import curses
import time

#from p20260130_12_kilo01.disk_scanner import scan_disks
#from p20260130_12_kilo01.ui import DiskManagerUI
from jusfltuls.disk_scanner import scan_disks
from jusfltuls.ui import DiskManagerUI


def run_ui(stdscr, refresh_interval: int = 10) -> None:
    """Main UI loop using curses.

    Args:
        stdscr: Curses window object
        refresh_interval: Auto-refresh interval in seconds
    """
    # Initialize UI
    ui = DiskManagerUI(stdscr)
    ui.left_panel.is_active = True

    # Initial disk scan
    disks, error = scan_disks()
    if error:
        ui.set_error(error)
    ui.update_disks(disks)

    # Load health info for all disks on startup
    for idx in range(len(disks)):
        ui.load_disk_health(idx)
    ui._update_left_panel()

    # Draw initial UI
    ui.draw()

    # Track last refresh time
    last_refresh = time.time()

    # Main loop - only redraw on user input or timer
    stdscr.nodelay(True)  # Non-blocking input
    while True:
        # Get user input (non-blocking)
        try:
            key = stdscr.getch()
        except KeyboardInterrupt:
            break

        # Check for auto-refresh
        current_time = time.time()
        needs_redraw = False

        if refresh_interval > 0 and (current_time - last_refresh) >= refresh_interval:
            # Auto-refresh: reload health info for all disks and update scrub status
            ui.set_status("refresh")
            for idx in range(len(ui.disks)):
                ui.load_disk_health(idx)
            ui._update_left_panel()
            ui.update_scrub_status()
            ui._update_all_partitions_panel()
            last_refresh = current_time
            needs_redraw = True

        # Also update scrub status on every keypress for responsiveness
        if key != -1:
            ui.update_scrub_status()

        # Handle input
        if key == ord('q') or key == ord('Q'):
            break
        elif key == ord('r') or key == ord('R'):
            # Manual refresh - reload health info for all disks to update test progress
            disks, error = scan_disks()
            if error:
                ui.set_error(error)
            else:
                ui.clear_messages()
            ui.update_disks(disks)
            # Reload health info for all disks to get updated test progress
            for idx in range(len(disks)):
                ui.load_disk_health(idx)
            ui._update_left_panel()
            needs_redraw = True
        elif key == ord('h') or key == ord('H'):
            # Load health info for selected disk
            if ui.active_panel == "left":
                selected_idx = ui.left_panel.selected_index
                if ui.load_disk_health(selected_idx):
                    ui._update_left_panel()
                    needs_redraw = True
        elif key == ord('t') or key == ord('T'):
            # Start short self-test for selected disk
            if ui.active_panel == "left":
                selected_idx = ui.left_panel.selected_index
                success, message = ui.start_short_test(selected_idx)
                if success:
                    ui.set_status(message)
                else:
                    ui.set_error(message)
                ui._update_left_panel()
                needs_redraw = True
        elif key == ord('s') or key == ord('S'):
            # Start btrfs scrub for selected partition in right panel
            if ui.active_panel == "right":
                selected_idx = ui.right_panel.selected_index
                success, message = ui.start_btrfs_scrub(selected_idx)
                if success:
                    ui.set_status(message)
                else:
                    ui.set_error(message)
                needs_redraw = True
        elif key == ord('\t') or key == 9:  # Tab key
            ui.switch_panel()
            needs_redraw = True
        elif key == curses.KEY_UP:
            ui.move_up()
            needs_redraw = True
        elif key == curses.KEY_DOWN:
            ui.move_down()
            needs_redraw = True
        elif key == curses.KEY_LEFT:
            if ui.active_panel == "right":
                ui.switch_panel()
                needs_redraw = True
        elif key == curses.KEY_RIGHT:
            if ui.active_panel == "left":
                ui.switch_panel()
                needs_redraw = True

        # Redraw when something changed
        if needs_redraw:
            ui.draw()

        # Small sleep to prevent CPU spinning
        time.sleep(0.1)


@click.command()
@click.option(
    "--refresh", "-r",
    default=10,
    help="Auto-refresh interval in seconds (0 to disable).",
    type=int
)
@click.version_option(version="0.1.0", prog_name="diskmanager")
def main(refresh: int) -> None:
    """
    Midnight Commander-style disk manager.

    Displays disks and partitions in a two-panel interface.
    Navigate with arrow keys, switch panels with Tab, quit with 'q'.
    """
    try:
        # Run the curses UI with refresh interval
        curses.wrapper(lambda stdscr: run_ui(stdscr, refresh))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


if __name__ == "__main__":
    main()



"""SMART health monitoring module."""

import re
from dataclasses import dataclass
from typing import Optional, Dict, Tuple
#from p20260130_12_kilo01.shell_calls import (
from jusfltuls.shell_calls import (
    build_sudo_smartctl_x,
    build_sudo_smartctl_l_selftest,
    build_sudo_smartctl_t_short,
    run_command,
)


@dataclass
class DiskHealth:
    """Represents the health status of a disk."""
    device: str
    overall_health: str  # "PASSED", "FAILED", or "UNKNOWN"
    last_test_status: str
    last_test_hours_ago: Optional[int]  # Hours since last test
    power_on_hours: Optional[int]
    model_name: Optional[str]
    test_in_progress: bool = False
    test_remaining_percent: Optional[int] = None  # % remaining if test in progress

    def __str__(self) -> str:
        health_str = f"{self.overall_health}"
        if self.test_in_progress and self.test_remaining_percent is not None:
            health_str += f" | Test: {self.test_remaining_percent}% done"
        elif self.last_test_hours_ago is not None:
            health_str += f" | Test: {self.last_test_hours_ago}h ago"
        return health_str


def parse_smartctl_output(output: str) -> DiskHealth:
    """Parse smartctl -x output to extract health information.

    Args:
        output: The raw output from smartctl -x command

    Returns:
        DiskHealth object with parsed information
    """
    device = "/dev/unknown"
    overall_health = "UNKNOWN"
    last_test_status = "No tests"
    last_test_hours_ago = None
    power_on_hours = None
    model_name = None
    test_in_progress = False
    test_remaining_percent = None

    lines = output.split('\n')

    # Parse device from first line if present
    if lines and '/dev/' in lines[0]:
        match = re.search(r'(/dev/\w+)', lines[0])
        if match:
            device = match.group(1)

    # Parse overall health
    for line in lines:
        if 'SMART overall-health self-assessment test result:' in line:
            if 'PASSED' in line:
                overall_health = "PASSED"
            elif 'FAILED' in line:
                overall_health = "FAILED"
            break

    # Parse model name
    for line in lines:
        if 'Device Model:' in line or 'Model Number:' in line:
            model_name = line.split(':', 1)[1].strip()
            break

    # Parse power on hours - handle both ATA and NVMe formats
    # ATA format: ID# ATTRIBUTE_NAME FLAGS VALUE WORST THRESH FAIL RAW_VALUE
    # Example: 9 Power_On_Hours -O--CK 091 091 000 - 44257
    # NVMe format: Power On Hours:                     6,576
    for line in lines:
        if 'Power_On_Hours' in line and not line.startswith('#'):
            parts = line.split()
            # Line format: 9 Power_On_Hours -O--CK 091 091 000 - 44257
            # Index:       0 1              2       3   4   5   6 7
            # RAW_VALUE is at index 7 (8th element)
            if len(parts) >= 8:
                try:
                    power_on_hours = int(parts[7])
                except (ValueError, IndexError):
                    pass
            break
        # NVMe format: "Power On Hours:                     6,576"
        elif 'Power On Hours:' in line:
            # Extract the number after the colon, handling comma separators
            match = re.search(r'Power On Hours:\s+([\d,]+)', line)
            if match:
                try:
                    # Remove commas and convert to int
                    power_on_hours = int(match.group(1).replace(',', ''))
                except (ValueError, IndexError):
                    pass
            break

    # Parse self-test execution status for in-progress tests
    for i, line in enumerate(lines):
        if 'Self-test execution status:' in line:
            # Check for in-progress status
            match = re.search(r'\(\s*(\d+)\s*\)', line)
            if match:
                status_code = int(match.group(1))
                # Status 0 = completed/no test, 240-249 = in progress
                if status_code >= 240 and status_code <= 249:
                    test_in_progress = True

            # Look for remaining percentage in this line or next few lines
            # Show percentage DONE (100 - remaining)
            for j in range(i, min(i + 3, len(lines))):
                remaining_match = re.search(r'(\d+)%\s+of\s+test\s+remaining', lines[j], re.IGNORECASE)
                if remaining_match:
                    remaining_percent = int(remaining_match.group(1))
                    test_remaining_percent = 100 - remaining_percent  # Convert to % done
                    break
            break

    # Also check for "Self-test status:" line (NVMe format)
    # Example: "Self-test status: Short self-test in progress (32% completed)"
    for line in lines:
        if 'Self-test status:' in line and 'in progress' in line.lower():
            test_in_progress = True
            # Extract percentage completed (already shows % done)
            completed_match = re.search(r'(\d+)%\s+completed', line, re.IGNORECASE)
            if completed_match:
                test_remaining_percent = int(completed_match.group(1))  # Already % done
            break

    # Parse self-test log for last completed test (only if no test in progress)
    if not test_in_progress:
        in_selftest_log = False
        for line in lines:
            if 'SMART Self-test log structure' in line:
                in_selftest_log = True
                continue

            if in_selftest_log:
                # Look for test entries like:
                # # 1  Short offline       Completed without error       00%     42191         -
                match = re.match(r'\s*#\s*1\s+(\S+)\s+(\S+)\s+(\S.*?)\s+(\d+)%\s+(\d+)', line)
                if match:
                    test_type = match.group(1)
                    test_status = match.group(3).strip()
                    lifetime_hours = int(match.group(5))

                    last_test_status = test_status

                    # Calculate hours ago if we have power on hours
                    if power_on_hours is not None:
                        last_test_hours_ago = power_on_hours - lifetime_hours
                    break

    return DiskHealth(
        device=device,
        overall_health=overall_health,
        last_test_status=last_test_status,
        last_test_hours_ago=last_test_hours_ago,
        power_on_hours=power_on_hours,
        model_name=model_name,
        test_in_progress=test_in_progress,
        test_remaining_percent=test_remaining_percent
    )


def parse_selftest_output(output: str, current_power_on_hours: Optional[int]) -> Tuple[str, Optional[int]]:
    """Parse smartctl -l selftest output to extract last test information.

    Args:
        output: The raw output from smartctl -l selftest command
        current_power_on_hours: Current power-on hours for calculating hours ago

    Returns:
        Tuple of (test_status, hours_ago)
    """
    lines = output.split('\n')
    last_test_status = "No tests"
    last_test_hours_ago = None

    # Find the header line to locate Power_on_Hours column position
    power_on_hours_col_start = None
    status_col_start = None
    for line in lines:
        if 'Power_on_Hours' in line or 'LifeTime(hours)' in line:
            # Found header line, get position of the column
            power_on_hours_col_start = line.find('Power_on_Hours')
            if power_on_hours_col_start == -1:
                power_on_hours_col_start = line.find('LifeTime(hours)')
            # Also find Status column position for NVMe
            status_col_start = line.find('Status')
            break

    for line in lines:
        # Skip header lines and empty lines
        if not line.strip() or line.startswith('Num') or line.startswith('SMART') or line.startswith('Self-test'):
            continue

        # Check if this is a test entry line (starts with a number)
        if not re.match(r'\s*#?\s*\d+', line):
            continue

        # For NVMe: extract Power_on_Hours from the column position
        if power_on_hours_col_start is not None and power_on_hours_col_start > 0:
            # Extract substring from Power_on_Hours column position
            hours_substring = line[power_on_hours_col_start:power_on_hours_col_start + 15]
            # Find first number in this substring
            hours_match = re.search(r'(\d+)', hours_substring)
            if hours_match:
                try:
                    lifetime_hours = int(hours_match.group(1))
                    # Extract test status from Status column
                    if status_col_start is not None and status_col_start > 0:
                        status_substring = line[status_col_start:power_on_hours_col_start]
                        # Clean up the status string
                        last_test_status = ' '.join(status_substring.split())
                    else:
                        last_test_status = "Unknown"

                    # Calculate hours ago if we have current power on hours
                    if current_power_on_hours is not None:
                        last_test_hours_ago = current_power_on_hours - lifetime_hours
                    break
                except (ValueError, IndexError):
                    pass
        else:
            # ATA format: # 1  Short offline       Completed without error       00%     4454         -
            match = re.match(r'\s*#?\s*(\d+)\s+(\S+)\s+(\S+)\s+(\S.*?)\s+(\d+)%?\s+(\d+)', line)
            if match:
                test_status = match.group(3).strip()
                lifetime_hours = int(match.group(6))

                last_test_status = test_status

                # Calculate hours ago if we have current power on hours
                if current_power_on_hours is not None:
                    last_test_hours_ago = current_power_on_hours - lifetime_hours
                break

    return last_test_status, last_test_hours_ago


class SmartHealthMonitor:
    """Main class for monitoring disk health via SMART."""

    def get_disk_health(self, device: str) -> DiskHealth:
        """Get health information for a disk using smartctl -x and -l selftest.

        Args:
            device: Device path (e.g., /dev/sda)

        Returns:
            DiskHealth object with health status
        """
        # Get basic health info from smartctl -x
        cmd = build_sudo_smartctl_x(device)
        returncode, output = run_command(cmd)

        # Parse output even if return code is non-zero
        # smartctl may return non-zero for USB devices but still provide valid data
        health = parse_smartctl_output(output)

        # If we successfully parsed a PASSED/FAILED status, use it
        # Otherwise, mark as ERROR only if we couldn't parse anything useful
        if health.overall_health == "UNKNOWN" and returncode != 0:
            health.overall_health = "ERROR"

        # Set the device path from the input
        health.device = device

        # Get self-test information from smartctl -l selftest
        selftest_cmd = build_sudo_smartctl_l_selftest(device)
        selftest_returncode, selftest_output = run_command(selftest_cmd)

        # Get self-test information from smartctl -l selftest
        # Parse output even if return code is non-zero (some devices return warnings)
        test_status, hours_ago = parse_selftest_output(
            selftest_output,
            health.power_on_hours
        )
        # Update if we found test information
        if test_status != "No tests":
            health.last_test_status = test_status
            health.last_test_hours_ago = hours_ago

        return health

    def get_all_disks_health(self, devices: list) -> Dict[str, DiskHealth]:
        """Get health information for multiple disks.

        Args:
            devices: List of device paths

        Returns:
            Dictionary mapping device paths to DiskHealth objects
        """
        result = {}
        for device in devices:
            result[device] = self.get_disk_health(device)
        return result

    def start_short_test(self, device: str) -> Tuple[bool, str]:
        """Start a short self-test on the given device.

        Args:
            device: Device path (e.g., /dev/sda)

        Returns:
            Tuple of (success, message)
        """
        cmd = build_sudo_smartctl_t_short(device)
        returncode, output = run_command(cmd)

        if returncode == 0:
            return True, "Short test started"
        else:
            return False, f"Failed to start test: {output}"
